@@ -28,7 +28,6 @@
 
 #include "goadaemon.h"
 #include "goabackend/goabackend.h"
-#include "goabackend/goaprovider-priv.h"
 #include "goabackend/goautils.h"
 
 struct _GoaDaemon
@@ -89,52 +88,6 @@ static void goa_daemon_reload_configuration (GoaDaemon *self);
 
 G_DEFINE_TYPE (GoaDaemon, goa_daemon, G_TYPE_OBJECT);
 
-/* ---------------------------------------------------------------------------------------------------- */
-
-typedef struct
-{
-  GError **error;
-  GList **out_providers;
-  GMainLoop *loop;
-  gboolean op_res;
-} GetAllSyncData;
-
-static void
-get_all_providers_sync_cb (GObject       *source_object,
-                           GAsyncResult  *res,
-                           gpointer       user_data)
-{
-  GetAllSyncData *data = (GetAllSyncData *) user_data;
-
-  data->op_res = goa_provider_get_all_finish (data->out_providers, res, data->error);
-  g_main_loop_quit (data->loop);
-}
-
-static gboolean
-get_all_providers_sync (GCancellable  *cancellable,
-                        GList        **out_providers,
-                        GError       **error)
-{
-  GetAllSyncData data;
-
-  data.error = error;
-  data.out_providers = out_providers;
-
-  /* HACK: Since telepathy-glib doesn't use the thread-default
-   * GMainContext for invoking the asynchronous callbacks, we can't
-   * push a new GMainContext here.
-   */
-  data.loop = g_main_loop_new (NULL, FALSE);
-
-  goa_provider_get_all (get_all_providers_sync_cb, &data);
-  g_main_loop_run (data.loop);
-  g_main_loop_unref (data.loop);
-
-  return data.op_res;
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
 static void
 goa_daemon_finalize (GObject *object)
 {
@@ -257,13 +210,30 @@ on_network_monitor_network_changed (GoaDaemon *self, gboolean available)
   queue_check_credentials (self);
 }
 
+#ifdef GOA_KERBEROS_ENABLED
+static void
+activate_identity_service (GoaDaemon *self)
+{
+  GoaProvider *provider;
+
+  /* We activate the identity service implicitly by using the kerberos
+   * backend.  This way if the kerberos backend isn't enabled, we don't
+   * end up starting the identity service needlessly
+   */
+  provider = goa_provider_get_for_provider_type (GOA_KERBEROS_NAME);
+
+  if (provider != NULL)
+    {
+      g_debug ("activated kerberos provider");
+      g_object_unref (provider);
+    }
+}
+#endif
+
 static void
 goa_daemon_init (GoaDaemon *self)
 {
   static volatile GQuark goa_error_domain = 0;
-  GError *error;
-  GList *l;
-  GList *providers = NULL;
   GoaObjectSkeleton *object;
   gchar *path;
 
@@ -273,22 +243,6 @@ goa_daemon_init (GoaDaemon *self)
   goa_error_domain = GOA_ERROR;
   goa_error_domain; /* shut up -Wunused-but-set-variable */
 
-  error = NULL;
-  if (!get_all_providers_sync (NULL, &providers, &error))
-    {
-      g_warning ("Unable to get the list of providers: %s (%s, %d)",
-                 error->message,
-                 g_quark_to_string (error->domain),
-                 error->code);
-      g_error_free (error);
-    }
-
-  for (l = providers; l != NULL; l = l->next)
-    {
-      GoaProvider *provider = GOA_PROVIDER (l->data);
-      goa_provider_initialize (provider);
-    }
-
   /* TODO: maybe nicer to pass in a GDBusConnection* construct property */
   self->connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
 
@@ -338,7 +292,9 @@ goa_daemon_init (GoaDaemon *self)
 
   queue_check_credentials (self);
 
-  g_list_free_full (providers, g_object_unref);
+#ifdef GOA_KERBEROS_ENABLED
+  activate_identity_service (self);
+#endif
 }
 
 static void
@@ -534,7 +490,6 @@ update_account_object (GoaDaemon           *self,
   GoaAccount *account;
   GoaProvider *provider;
   gboolean is_locked;
-  gboolean is_temporary;
   gboolean ret;
   gchar *identity;
   gchar *presentation_identity;
@@ -565,7 +520,6 @@ update_account_object (GoaDaemon           *self,
   identity = g_key_file_get_string (key_file, group, "Identity", NULL);
   presentation_identity = g_key_file_get_string (key_file, group, "PresentationIdentity", NULL);
   is_locked = g_key_file_get_boolean (key_file, group, "IsLocked", NULL);
-  is_temporary = g_key_file_get_boolean (key_file, group, "IsTemporary", NULL);
   if (just_added)
     {
       account = goa_account_skeleton_new ();
@@ -588,7 +542,6 @@ update_account_object (GoaDaemon           *self,
   goa_account_set_identity (account, identity);
   goa_account_set_presentation_identity (account, presentation_identity);
   goa_account_set_is_locked (account, is_locked);
-  goa_account_set_is_temporary (account, is_temporary);
 
   error = NULL;
   if (!goa_provider_build_object (provider, object, key_file, group, self->connection, just_added, &error))
@@ -958,8 +911,23 @@ get_all_providers_cb (GObject      *source,
       g_key_file_set_string (key_file, group, key, value);
     }
 
+  g_free (key_file_data);
   error = NULL;
-  if (!g_key_file_save_to_file (key_file, path, &error))
+  key_file_data = g_key_file_to_data (key_file,
+                                      &length,
+                                      &error);
+  if (key_file_data == NULL)
+    {
+      g_prefix_error (&error, "Error generating key-value-file: ");
+      g_dbus_method_invocation_take_error (data->invocation, error);
+      goto out;
+    }
+
+  error = NULL;
+  if (!g_file_set_contents (path,
+                            key_file_data,
+                            length,
+                            &error))
     {
       g_prefix_error (&error, "Error writing key-value-file %s: ", path);
       g_dbus_method_invocation_take_error (data->invocation, error);
@@ -1029,80 +997,19 @@ on_manager_handle_add_account (GoaManager             *manager,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-typedef struct
-{
-  GoaObject *object;
-  GList *invocations;
-} ObjectInvocationData;
-
-static ObjectInvocationData *
-object_invocation_data_new (GoaObject             *object,
-                 GDBusMethodInvocation *invocation)
-{
-  ObjectInvocationData *data;
-  data = g_slice_new0 (ObjectInvocationData);
-  data->object = g_object_ref (object);
-  data->invocations = g_list_prepend (data->invocations, invocation);
-  return data;
-}
-
-static void
-object_invocation_data_unref (ObjectInvocationData *data)
-{
-  g_list_free (data->invocations);
-  g_object_unref (data->object);
-  g_slice_free (ObjectInvocationData, data);
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
-static void
-remove_account_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  GTask *task = G_TASK (user_data);
-  GoaDaemon *self;
-  GDBusMethodInvocation *invocation;
-  GError *error;
-  GoaAccount *account;
-  GoaProvider *provider = GOA_PROVIDER (source_object);
-  ObjectInvocationData *data;
-
-  self = GOA_DAEMON (g_task_get_source_object (task));
-  data = g_task_get_task_data (task);
-
-  error= NULL;
-  if (!goa_provider_remove_account_finish (provider, res, &error))
-    {
-      g_warning ("goa_provider_remove_account() failed: %s (%s, %d)",
-                 error->message,
-                 g_quark_to_string (error->domain),
-                 error->code);
-      g_error_free (error);
-    }
-
-  goa_daemon_reload_configuration (self);
-
-  account = goa_object_peek_account (data->object);
-  invocation = G_DBUS_METHOD_INVOCATION (data->invocations->data);
-  goa_account_complete_remove (account, invocation);
-
-  g_object_unref (task);
-}
-
 static gboolean
 on_account_handle_remove (GoaAccount            *account,
                           GDBusMethodInvocation *invocation,
                           gpointer               user_data)
 {
   GoaDaemon *self = GOA_DAEMON (user_data);
-  GoaObject *object;
   GoaProvider *provider;
   GKeyFile *key_file;
-  GTask *task;
-  ObjectInvocationData *data;
   const gchar *provider_type;
   gchar *path;
   gchar *group;
+  gchar *data;
+  gsize length;
   GError *error;
 
   provider = NULL;
@@ -1110,7 +1017,7 @@ on_account_handle_remove (GoaAccount            *account,
   path = NULL;
   group = NULL;
   key_file = NULL;
-  task = NULL;
+  data = NULL;
 
   if (goa_account_get_is_locked (account))
     {
@@ -1150,7 +1057,21 @@ on_account_handle_remove (GoaAccount            *account,
     }
 
   error = NULL;
-  if (!g_key_file_save_to_file (key_file, path, &error))
+  data = g_key_file_to_data (key_file,
+                             &length,
+                             &error);
+  if (data == NULL)
+    {
+      g_prefix_error (&error, "Error generating key-value-file: ");
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
+  error = NULL;
+  if (!g_file_set_contents (path,
+                            data,
+                            length,
+                            &error))
     {
       g_prefix_error (&error, "Error writing key-value-file %s: ", path);
       g_dbus_method_invocation_take_error (invocation, error);
@@ -1189,17 +1110,13 @@ on_account_handle_remove (GoaAccount            *account,
       goto out;
     }
 
-  object = GOA_OBJECT (g_dbus_interface_get_object (G_DBUS_INTERFACE (account)));
-  data = object_invocation_data_new (object, invocation);
+  goa_daemon_reload_configuration (self);
 
-  task = g_task_new (self, NULL, NULL, NULL);
-  g_task_set_task_data (task, data, (GDestroyNotify) object_invocation_data_unref);
-
-  goa_provider_remove_account (provider, object, NULL, remove_account_cb, g_object_ref (task));
+  goa_account_complete_remove (account, invocation);
 
  out:
   g_clear_object (&provider);
-  g_clear_object (&task);
+  g_free (data);
   g_clear_pointer (&key_file, (GDestroyNotify) g_key_file_free);
   g_free (group);
   g_free (path);
@@ -1208,6 +1125,31 @@ on_account_handle_remove (GoaAccount            *account,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+typedef struct
+{
+  GoaObject *object;
+  GList *invocations;
+} EnsureData;
+
+static EnsureData *
+ensure_data_new (GoaObject             *object,
+                 GDBusMethodInvocation *invocation)
+{
+  EnsureData *data;
+  data = g_slice_new0 (EnsureData);
+  data->object = g_object_ref (object);
+  data->invocations = g_list_prepend (data->invocations, invocation);
+  return data;
+}
+
+static void
+ensure_data_unref (EnsureData *data)
+{
+  g_list_free (data->invocations);
+  g_object_unref (data->object);
+  g_slice_free (EnsureData, data);
+}
+
 static gboolean
 is_authorization_error (GError *error)
 {
@@ -1260,18 +1202,15 @@ static void
 ensure_credentials_queue_collector (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   GTask *task = G_TASK (user_data);
-  GTask *task_queued;
   GoaDaemon *self;
+  EnsureData *data;
   GoaAccount *account;
   GoaProvider *provider = GOA_PROVIDER (source_object);
   GError *error;
-  ObjectInvocationData *data;
   gint expires_in;
 
   self = GOA_DAEMON (g_task_get_source_object (task));
-
-  task_queued = G_TASK (g_queue_pop_head (self->ensure_credentials_queue));
-  g_assert (task == task_queued);
+  g_assert (task == G_TASK (g_queue_pop_head (self->ensure_credentials_queue)));
 
   data = g_task_get_task_data (task);
   account = goa_object_peek_account (data->object);
@@ -1331,10 +1270,10 @@ ensure_credentials_queue_sort (gconstpointer a, gconstpointer b, gpointer user_d
 static void
 ensure_credentials_queue_check (GoaDaemon *self)
 {
+  EnsureData *data;
   GoaAccount *account;
   GoaProvider *provider = NULL;
   GTask *task;
-  ObjectInvocationData *data;
   const gchar *id;
   const gchar *provider_type;
   gint64 timestamp;
@@ -1350,7 +1289,7 @@ ensure_credentials_queue_check (GoaDaemon *self)
   task = G_TASK (g_queue_peek_head (self->ensure_credentials_queue));
   self->ensure_credentials_running = TRUE;
 
-  data = (ObjectInvocationData *) g_task_get_task_data (task);
+  data = (EnsureData *) g_task_get_task_data (task);
   account = goa_object_peek_account (data->object);
 
   id = goa_account_get_id (account);
@@ -1389,7 +1328,7 @@ ensure_credentials_queue_coalesce (GoaDaemon *self, GoaObject *object, GDBusMeth
     {
       GoaAccount *account_queued;
       GTask *task = G_TASK (l->data);
-      ObjectInvocationData *data;
+      EnsureData *data;
       const gchar *id_queued;
 
       data = g_task_get_task_data (task);
@@ -1419,10 +1358,10 @@ on_account_handle_ensure_credentials (GoaAccount            *account,
                                       gpointer               user_data)
 {
   GoaDaemon *self = GOA_DAEMON (user_data);
+  EnsureData *data;
   GoaObject *object;
   GoaProvider *provider = NULL;
   GTask *task = NULL;
-  ObjectInvocationData *data;
   const gchar *id;
   const gchar *method_name;
   const gchar *provider_type;
@@ -1458,11 +1397,11 @@ on_account_handle_ensure_credentials (GoaAccount            *account,
       goto out;
     }
 
-  data = object_invocation_data_new (object, invocation);
+  data = ensure_data_new (object, invocation);
 
   task = g_task_new (self, NULL, NULL, NULL);
   g_task_set_priority (task, G_PRIORITY_HIGH);
-  g_task_set_task_data (task, data, (GDestroyNotify) object_invocation_data_unref);
+  g_task_set_task_data (task, data, (GDestroyNotify) ensure_data_unref);
   g_queue_push_tail (self->ensure_credentials_queue, g_object_ref (task));
 
   ensure_credentials_queue_check (self);
@@ -1490,23 +1429,23 @@ goa_daemon_check_credentials (GoaDaemon *self)
   objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (self->object_manager));
   for (l = objects; l != NULL; l = l->next)
     {
+      EnsureData *data;
       GoaAccount *account;
       GoaObject *object = GOA_OBJECT (l->data);
       GoaProvider *provider = NULL;
       GTask *task = NULL;
-      ObjectInvocationData *data;
       const gchar *id;
       const gchar *provider_type;
       gint64 timestamp;
 
       account = goa_object_peek_account (object);
       if (account == NULL)
-        goto cleanup_and_continue;
+        continue;
 
       provider_type = goa_account_get_provider_type (account);
       provider = goa_provider_get_for_provider_type (provider_type);
       if (provider == NULL)
-        goto cleanup_and_continue;
+        continue;
 
       id = goa_account_get_id (account);
       provider_type = goa_account_get_provider_type (account);
@@ -1523,17 +1462,16 @@ goa_daemon_check_credentials (GoaDaemon *self)
                    timestamp,
                    provider_type,
                    id);
-          goto cleanup_and_continue;
+          continue;
         }
 
-      data = object_invocation_data_new (object, NULL);
+      data = ensure_data_new (object, NULL);
 
       task = g_task_new (self, NULL, NULL, NULL);
       g_task_set_priority (task, G_PRIORITY_LOW);
-      g_task_set_task_data (task, data, (GDestroyNotify) object_invocation_data_unref);
+      g_task_set_task_data (task, data, (GDestroyNotify) ensure_data_unref);
       g_queue_push_tail (self->ensure_credentials_queue, g_object_ref (task));
 
-    cleanup_and_continue:
       g_clear_object (&provider);
       g_clear_object (&task);
     }
