@@ -952,9 +952,6 @@ add_account (GoaProvider    *provider,
       goto out;
     }
 
-  gtk_widget_set_no_show_all (data.cluebar, TRUE);
-  gtk_widget_hide (data.cluebar);
-
   encryption = gtk_combo_box_get_active_id (GTK_COMBO_BOX (data.imap_encryption));
   imap_tls_type = get_tls_type_from_string_id (encryption);
 
@@ -1024,8 +1021,7 @@ add_account (GoaProvider    *provider,
 
   /* SMTP */
 
-  /* Re-use the username and password from the IMAP page */
-  gtk_entry_set_text (GTK_ENTRY (data.smtp_username), imap_username);
+  /* Re-use the password from the IMAP page */
   gtk_entry_set_text (GTK_ENTRY (data.smtp_password), imap_password);
 
   gtk_notebook_next_page (GTK_NOTEBOOK (data.notebook));
@@ -1044,9 +1040,6 @@ add_account (GoaProvider    *provider,
       goto out;
     }
 
-  gtk_widget_set_no_show_all (data.cluebar, TRUE);
-  gtk_widget_hide (data.cluebar);
-
   encryption = gtk_combo_box_get_active_id (GTK_COMBO_BOX (data.smtp_encryption));
   smtp_tls_type = get_tls_type_from_string_id (encryption);
 
@@ -1112,7 +1105,6 @@ add_account (GoaProvider    *provider,
       gtk_widget_show_all (data.cluebar);
 
       g_clear_object (&smtp_auth);
-      g_clear_pointer (&domain, g_free);
       goto smtp_again;
     }
 
@@ -1311,9 +1303,6 @@ refresh_account (GoaProvider    *provider,
       goto out;
     }
 
-  gtk_widget_set_no_show_all (data.cluebar, TRUE);
-  gtk_widget_hide (data.cluebar);
-
   imap_password = gtk_entry_get_text (GTK_ENTRY (data.imap_password));
   g_cancellable_reset (data.cancellable);
   imap_auth = goa_imap_auth_login_new (NULL, NULL, imap_username, imap_password);
@@ -1387,9 +1376,6 @@ refresh_account (GoaProvider    *provider,
       goto out;
     }
 
-  gtk_widget_set_no_show_all (data.cluebar, TRUE);
-  gtk_widget_hide (data.cluebar);
-
   smtp_password = gtk_entry_get_text (GTK_ENTRY (data.smtp_password));
   g_cancellable_reset (data.cancellable);
   goa_utils_parse_email_address (email_address, NULL, &domain);
@@ -1435,7 +1421,6 @@ refresh_account (GoaProvider    *provider,
       gtk_widget_show_all (data.cluebar);
 
       g_clear_object (&smtp_auth);
-      g_clear_pointer (&domain, g_free);
       goto smtp_again;
     }
 
@@ -1496,10 +1481,10 @@ show_label (GtkWidget *grid, gint row, const gchar *left, const gchar *right)
   gtk_grid_attach (GTK_GRID (grid), label, 0, row, 1, 1);
 
   label = gtk_label_new (right);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
   gtk_label_set_max_width_chars (GTK_LABEL (label), 24);
   gtk_label_set_width_chars (GTK_LABEL (label), 24);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
   gtk_grid_attach (GTK_GRID (grid), label, 1, row, 3, 1);
 }
 
