@@ -85,7 +85,6 @@ goa_kerberos_provider_module_init (void)
 {
   create_object_manager ();
   create_identity_manager ();
-  g_debug ("activated kerberos provider");
 }
 
 static const gchar *
@@ -244,8 +243,6 @@ sign_in_identity (GoaKerberosProvider  *self,
                                        sign_in_thread,
                                        G_PRIORITY_DEFAULT,
                                        cancellable);
-
-  g_object_unref (operation_result);
 }
 
 static void
@@ -352,8 +349,7 @@ get_ticket_sync (GoaKerberosProvider *self,
     }
 
   ret = TRUE;
-
- out:
+out:
   g_clear_object (&account);
   g_clear_object (&ticketing);
   g_free (object_path);
@@ -845,7 +841,7 @@ on_initial_sign_in_done (GoaKerberosProvider *self,
   else if (remember_password)
     {
       GVariantBuilder  builder;
-      char            *object_path;
+      const char      *object_path;
 
       object_path = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result));
 
@@ -873,8 +869,6 @@ on_initial_sign_in_done (GoaKerberosProvider *self,
                                                        NULL,
                                                        NULL);
         }
-
-      g_free (object_path);
     }
 
   g_simple_async_result_complete_in_idle (operation_result);
@@ -1323,16 +1317,13 @@ static gboolean
 dbus_proxy_reload_properties_sync (GDBusProxy    *proxy,
                                    GCancellable  *cancellable)
 {
-  GVariant *result = NULL;
+  GVariant *result;
   char *name;
-  char *name_owner = NULL;
   GVariant *value;
-  GVariantIter *iter = NULL;
-  gboolean ret = FALSE;
+  GVariantIter *iter;
 
-  name_owner = g_dbus_proxy_get_name_owner (proxy);
   result = g_dbus_connection_call_sync (g_dbus_proxy_get_connection (proxy),
-                                        name_owner,
+                                        g_dbus_proxy_get_name_owner (proxy),
                                         g_dbus_proxy_get_object_path (proxy),
                                         "org.freedesktop.DBus.Properties",
                                         "GetAll",
@@ -1343,7 +1334,7 @@ dbus_proxy_reload_properties_sync (GDBusProxy    *proxy,
                                         cancellable,
                                         NULL);
   if (result == NULL)
-    goto out;
+    return FALSE;
 
   g_variant_get (result, "(a{sv})", &iter);
   while (g_variant_iter_next (iter, "{sv}", &name, &value))
@@ -1353,14 +1344,8 @@ dbus_proxy_reload_properties_sync (GDBusProxy    *proxy,
       g_free (name);
       g_variant_unref (value);
     }
-
-  ret = TRUE;
-
- out:
-  g_clear_pointer (&iter, (GDestroyNotify) g_variant_iter_free);
-  g_clear_pointer (&result, (GDestroyNotify) g_variant_unref);
-  g_free (name_owner);
-  return ret;
+  g_variant_iter_free (iter);
+  return TRUE;
 }
 
 static gboolean
@@ -1448,83 +1433,13 @@ ensure_credentials_sync (GoaProvider    *provider,
   g_date_time_unref (now);
   g_date_time_unref (expiration_time);
 
- out:
+out:
   g_clear_object (&account);
   g_clear_object (&identity);
   g_mutex_unlock (&identity_manager_mutex);
   return credentials_ensured;
 }
 
-/* ---------------------------------------------------------------------------------------------------- */
-
-static void
-remove_account_in_thread_func (GTask         *task,
-                               gpointer       source_object,
-                               gpointer       task_data,
-                               GCancellable  *cancellable)
-{
-  GError *error;
-  GoaAccount *account = NULL;
-  GoaObject *object = GOA_OBJECT (task_data);
-  const gchar *identifier;
-
-  ensure_identity_manager ();
-
-  account = goa_object_get_account (object);
-  identifier = goa_account_get_identity (account);
-
-  g_debug ("Kerberos account %s removed and should now be signed out", identifier);
-
-  error = NULL;
-  if (!goa_identity_service_manager_call_sign_out_sync (identity_manager, identifier, cancellable, &error))
-    {
-      g_task_return_error (task, error);
-      goto out;
-    }
-
-  g_task_return_boolean (task, TRUE);
-
- out:
-  g_clear_object (&account);
-}
-
-static void
-remove_account (GoaProvider          *provider,
-                GoaObject            *object,
-                GCancellable         *cancellable,
-                GAsyncReadyCallback   callback,
-                gpointer              user_data)
-{
-  GoaKerberosProvider *self = GOA_KERBEROS_PROVIDER (provider);
-  GTask *task;
-
-  task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_source_tag (task, remove_account);
-
-  g_task_set_task_data (task, g_object_ref (object), g_object_unref);
-  g_task_run_in_thread (task, remove_account_in_thread_func);
-
-  g_object_unref (task);
-}
-
-static gboolean
-remove_account_finish (GoaProvider   *provider,
-                       GAsyncResult  *res,
-                       GError       **error)
-{
-  GoaKerberosProvider *self = GOA_KERBEROS_PROVIDER (provider);
-  GTask *task;
-
-  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
-  task = G_TASK (res);
-
-  g_warn_if_fail (g_task_get_source_tag (task) == remove_account);
-
-  return g_task_propagate_boolean (task, error);
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
 static GoaIdentityServiceIdentity *
 get_identity_from_object_manager (GoaKerberosProvider *self,
                                   const char          *identifier)
@@ -1577,7 +1492,7 @@ sign_in_identity_sync (GoaKerberosProvider  *self,
 {
   GcrSecretExchange  *secret_exchange;
   char               *secret_key;
-  char               *return_key = NULL;
+  char               *return_key;
   char               *concealed_secret;
   char               *identity_object_path = NULL;
   gboolean            keys_exchanged;
@@ -1630,8 +1545,7 @@ sign_in_identity_sync (GoaKerberosProvider  *self,
                                                   error);
   g_mutex_unlock (&identity_manager_mutex);
 
- out:
-  g_free (return_key);
+out:
   g_object_unref (secret_exchange);
   return identity_object_path;
 }
@@ -1653,6 +1567,7 @@ sign_in_thread (GSimpleAsyncResult  *result,
 
   error = NULL;
   object_path = sign_in_identity_sync (self, identifier, password, preauth_source, cancellable, &error);
+
   if (object_path == NULL)
     g_simple_async_result_take_error (result, error);
   else
@@ -1769,6 +1684,4 @@ goa_kerberos_provider_class_init (GoaKerberosProviderClass *kerberos_class)
   provider_class->add_account                = add_account;
   provider_class->refresh_account            = refresh_account;
   provider_class->ensure_credentials_sync    = ensure_credentials_sync;
-  provider_class->remove_account             = remove_account;
-  provider_class->remove_account_finish      = remove_account_finish;
 }
