@@ -81,19 +81,6 @@ static guint goa_provider_get_credentials_generation_real (GoaProvider *self);
 
 static GIcon *goa_provider_get_provider_icon_real (GoaProvider *self,
                                                    GoaObject   *object);
-
-static void goa_provider_initialize_real (GoaProvider *self);
-
-static void goa_provider_remove_account_real (GoaProvider          *self,
-                                              GoaObject            *object,
-                                              GCancellable         *cancellable,
-                                              GAsyncReadyCallback   callback,
-                                              gpointer              user_data);
-
-static gboolean goa_provider_remove_account_finish_real (GoaProvider   *self,
-                                                         GAsyncResult  *res,
-                                                         GError       **error);
-
 static void goa_provider_show_account_real (GoaProvider         *provider,
                                             GoaClient           *client,
                                             GoaObject           *object,
@@ -105,82 +92,6 @@ static void goa_provider_show_account_real (GoaProvider         *provider,
 
 G_DEFINE_ABSTRACT_TYPE (GoaProvider, goa_provider, G_TYPE_OBJECT);
 
-static struct {
-  GoaProviderFeatures feature;
-  const gchar *property;
-  const gchar *blurb;
-} provider_features_info[] = {
-  /* The order in which the features are listed is
-   * important because it affects the order in which they are
-   * displayed in the show_account() UI
-   */
-  {
-    .feature = GOA_PROVIDER_FEATURE_MAIL,
-    .property = "mail-disabled",
-    .blurb = N_("_Mail"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_CALENDAR,
-    .property = "calendar-disabled",
-    .blurb = N_("Cale_ndar"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_CONTACTS,
-    .property = "contacts-disabled",
-    .blurb = N_("_Contacts"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_CHAT,
-    .property = "chat-disabled",
-    .blurb = N_("C_hat"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_DOCUMENTS,
-    .property = "documents-disabled",
-    .blurb = N_("_Documents"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_MUSIC,
-    .property = "music-disabled",
-    .blurb = N_("M_usic"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_PHOTOS,
-    .property = "photos-disabled",
-    .blurb = N_("_Photos"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_FILES,
-    .property = "files-disabled",
-    .blurb = N_("_Files"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_TICKETING,
-    .property = "ticketing-disabled",
-    .blurb = N_("Network _Resources"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_READ_LATER,
-    .property = "read-later-disabled",
-    .blurb = N_("_Read Later"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_PRINTERS,
-    .property = "printers-disabled",
-    .blurb = N_("Prin_ters"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_MAPS,
-    .property = "maps-disabled",
-    .blurb = N_("_Maps"),
-  },
-  {
-    .feature = GOA_PROVIDER_FEATURE_INVALID,
-    .property = NULL,
-    .blurb = NULL,
-  }
-};
-
 static void
 goa_provider_get_property (GObject *object,
                            guint property_id,
@@ -248,9 +159,6 @@ goa_provider_class_init (GoaProviderClass *klass)
   klass->ensure_credentials_sync = goa_provider_ensure_credentials_sync_real;
   klass->get_credentials_generation = goa_provider_get_credentials_generation_real;
   klass->get_provider_icon = goa_provider_get_provider_icon_real;
-  klass->initialize = goa_provider_initialize_real;
-  klass->remove_account = goa_provider_remove_account_real;
-  klass->remove_account_finish = goa_provider_remove_account_finish_real;
   klass->show_account = goa_provider_show_account_real;
 
 /**
@@ -418,21 +326,6 @@ goa_provider_get_provider_features (GoaProvider *self)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-void
-goa_provider_initialize (GoaProvider *self)
-{
-  g_return_if_fail (GOA_IS_PROVIDER (self));
-
-  GOA_PROVIDER_GET_CLASS (self)->initialize (self);
-}
-
-static void
-goa_provider_initialize_real (GoaProvider *self)
-{
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
 /**
  * goa_provider_add_account:
  * @self: A #GoaProvider.
@@ -571,6 +464,82 @@ goa_provider_show_account (GoaProvider         *self,
   GOA_PROVIDER_GET_CLASS (self)->show_account (self, client, object, vbox, grid, dummy);
 }
 
+static struct {
+  GoaProviderFeatures feature;
+  const gchar *property;
+  const gchar *blurb;
+} show_account_items[] = {
+  /* The order in which the features are listed is
+   * important because it affects the order in which they are
+   * displayed in the show_account() UI
+   */
+  {
+    .feature = GOA_PROVIDER_FEATURE_MAIL,
+    .property = "mail-disabled",
+    .blurb = N_("_Mail"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_CALENDAR,
+    .property = "calendar-disabled",
+    .blurb = N_("Cale_ndar"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_CONTACTS,
+    .property = "contacts-disabled",
+    .blurb = N_("_Contacts"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_CHAT,
+    .property = "chat-disabled",
+    .blurb = N_("C_hat"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_DOCUMENTS,
+    .property = "documents-disabled",
+    .blurb = N_("_Documents"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_MUSIC,
+    .property = "music-disabled",
+    .blurb = N_("M_usic"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_PHOTOS,
+    .property = "photos-disabled",
+    .blurb = N_("_Photos"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_FILES,
+    .property = "files-disabled",
+    .blurb = N_("_Files"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_TICKETING,
+    .property = "ticketing-disabled",
+    .blurb = N_("Network _Resources"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_READ_LATER,
+    .property = "read-later-disabled",
+    .blurb = N_("_Read Later"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_PRINTERS,
+    .property = "printers-disabled",
+    .blurb = N_("Prin_ters"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_MAPS,
+    .property = "maps-disabled",
+    .blurb = N_("_Maps"),
+  },
+  {
+    .feature = GOA_PROVIDER_FEATURE_INVALID,
+    .property = NULL,
+    .blurb = NULL,
+  }
+};
+
 static void
 goa_provider_show_account_real (GoaProvider         *provider,
                                 GoaClient           *client,
@@ -593,14 +562,14 @@ goa_provider_show_account_real (GoaProvider         *provider,
    * options switches. For example: “Use for Mail”. */
   label = _("Use for");
 
-  for (i = 0; provider_features_info[i].property != NULL; i++)
+  for (i = 0; show_account_items[i].property != NULL; i++)
     {
-      if ((features & provider_features_info[i].feature) != 0)
+      if ((features & show_account_items[i].feature) != 0)
         {
           goa_util_add_row_switch_from_keyfile_with_blurb (grid, row++, object,
                                                            label,
-                                                           provider_features_info[i].property,
-                                                           _(provider_features_info[i].blurb));
+                                                           show_account_items[i].property,
+                                                           _(show_account_items[i].blurb));
           label = NULL;
         }
     }
@@ -683,14 +652,15 @@ ensure_credentials_data_free (EnsureCredentialsData *data)
 }
 
 static void
-ensure_credentials_in_thread_func (GSimpleAsyncResult *simple,
-                                   GObject            *object,
+ensure_credentials_in_thread_func (GTask              *task,
+                                   gpointer            object,
+                                   gpointer            task_data,
                                    GCancellable       *cancellable)
 {
   EnsureCredentialsData *data;
   GError *error;
 
-  data = g_simple_async_result_get_op_res_gpointer (simple);
+  data = task_data;
 
   error = NULL;
   if (!goa_provider_ensure_credentials_sync (GOA_PROVIDER (object),
@@ -698,7 +668,9 @@ ensure_credentials_in_thread_func (GSimpleAsyncResult *simple,
                                              &data->expires_in,
                                              cancellable,
                                              &error))
-    g_simple_async_result_take_error (simple, error);
+    g_task_return_error (task, error);
+  else
+    g_task_return_boolean (task, TRUE);
 }
 
 
@@ -729,24 +701,23 @@ goa_provider_ensure_credentials (GoaProvider          *self,
                                  GAsyncReadyCallback   callback,
                                  gpointer              user_data)
 {
-  GSimpleAsyncResult *simple;
+  GTask *task;
 
   g_return_if_fail (GOA_IS_PROVIDER (self));
   g_return_if_fail (GOA_IS_OBJECT (object));
   g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
-  simple = g_simple_async_result_new (G_OBJECT (self),
-                                      callback,
-                                      user_data,
-                                      goa_provider_ensure_credentials);
-  g_simple_async_result_set_op_res_gpointer (simple,
-                                             ensure_credentials_data_new (object),
-                                             (GDestroyNotify) ensure_credentials_data_free);
-  g_simple_async_result_run_in_thread (simple,
-                                       ensure_credentials_in_thread_func,
-                                       G_PRIORITY_DEFAULT,
-                                       cancellable);
-  g_object_unref (simple);
+  task = g_task_new (self,
+                     cancellable,
+                     callback,
+                     user_data);
+  g_task_set_task_data (task,
+                        ensure_credentials_data_new (object),
+                        (GDestroyNotify) ensure_credentials_data_free);
+  g_task_set_source_tag (task, goa_provider_ensure_credentials);
+  g_task_run_in_thread (task, ensure_credentials_in_thread_func);
+
+  g_object_unref (task);
 }
 
 /**
@@ -761,28 +732,32 @@ goa_provider_ensure_credentials (GoaProvider          *self,
  * Returns: %TRUE if the credentials for the passed #GoaObject are valid, %FALSE if @error is set.
  */
 gboolean
-goa_provider_ensure_credentials_finish (GoaProvider  *self,
-                                                gint                *out_expires_in,
-                                                GAsyncResult        *res,
-                                                GError             **error)
+goa_provider_ensure_credentials_finish (GoaProvider         *self,
+                                        gint                *out_expires_in,
+                                        GAsyncResult        *res,
+                                        GError             **error)
 {
-  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
+  GTask *task = G_TASK (res);
+  gboolean had_error;
+
   gboolean ret;
   EnsureCredentialsData *data;
 
   ret = FALSE;
 
   g_return_val_if_fail (GOA_IS_PROVIDER (self), FALSE);
-  g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == goa_provider_ensure_credentials);
+  g_warn_if_fail (g_task_get_source_tag (task) == goa_provider_ensure_credentials);
 
-  if (g_simple_async_result_propagate_error (simple, error))
+  /* Workaround for bgo#764163 */
+  had_error = g_task_had_error (task);
+  ret = g_task_propagate_boolean (task, error);
+  if (had_error)
     goto out;
 
-  ret = TRUE;
-  data = g_simple_async_result_get_op_res_gpointer (simple);
+  data = g_task_get_task_data (task);
   if (out_expires_in != NULL)
     *out_expires_in = data->expires_in;
 
@@ -812,46 +787,10 @@ goa_provider_ensure_credentials_sync (GoaProvider     *self,
                                       GCancellable    *cancellable,
                                       GError         **error)
 {
-  GoaAccount *account = NULL;
-  GoaProviderFeatures features;
-  gboolean disabled = TRUE;
-  gboolean ret = FALSE;
-  guint i;
-
   g_return_val_if_fail (GOA_IS_PROVIDER (self), FALSE);
-  g_return_val_if_fail (GOA_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-  account = goa_object_get_account (object);
-  g_return_val_if_fail (GOA_IS_ACCOUNT (account), FALSE);
-
-  features = goa_provider_get_provider_features (self);
-
-  for (i = 0; provider_features_info[i].property != NULL; i++)
-    {
-      if ((features & provider_features_info[i].feature) != 0)
-        {
-          gboolean feature_disabled;
-
-          g_object_get (account, provider_features_info[i].property, &feature_disabled, NULL);
-          disabled = disabled && feature_disabled;
-          if (!disabled)
-            break;
-        }
-    }
-
-  if (disabled)
-    {
-      g_set_error_literal (error, GOA_ERROR, GOA_ERROR_NOT_SUPPORTED, _("Account is disabled"));
-      goto out;
-    }
-
-  ret = GOA_PROVIDER_GET_CLASS (self)->ensure_credentials_sync (self, object, out_expires_in, cancellable, error);
-
- out:
-  g_clear_object (&account);
-  return ret;
+  return GOA_PROVIDER_GET_CLASS (self)->ensure_credentials_sync (self, object, out_expires_in, cancellable, error);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -992,8 +931,8 @@ static struct
   { NULL, NULL }
 };
 
-void
-goa_provider_ensure_builtins_loaded (void)
+static void
+ensure_builtins_loaded (void)
 {
   static gsize once_init_value = 0;
 
@@ -1082,7 +1021,7 @@ goa_provider_get_for_provider_type (const gchar *provider_type)
 
   g_return_val_if_fail (provider_type != NULL, NULL);
 
-  goa_provider_ensure_builtins_loaded ();
+  ensure_builtins_loaded ();
 
   ret = NULL;
 
@@ -1120,7 +1059,7 @@ typedef struct
 {
   GQueue ret;
   gint pending_calls;
-  GSimpleAsyncResult *result;
+  GTask *task;
 } GetAllData;
 
 static void
@@ -1167,11 +1106,9 @@ get_all_check_done (GetAllData *data)
   g_queue_sort (&data->ret, (GCompareDataFunc) compare_providers, NULL);
 
   /* Steal the list out of the GQueue. */
-  g_simple_async_result_set_op_res_gpointer (data->result, data->ret.head,
-      free_list_and_unref);
-  g_simple_async_result_complete_in_idle (data->result);
+  g_task_return_pointer (data->task, data->ret.head, free_list_and_unref);
 
-  g_object_unref (data->result);
+  g_object_unref (data->task);
   g_slice_free (GetAllData, data);
 }
 
@@ -1234,11 +1171,11 @@ goa_provider_get_all (GAsyncReadyCallback callback,
   GetAllData *data;
   gint i;
 
-  goa_provider_ensure_builtins_loaded ();
+  ensure_builtins_loaded ();
 
   data = g_slice_new0 (GetAllData);
-  data->result = g_simple_async_result_new (NULL, callback, user_data,
-      goa_provider_get_all);
+  data->task = g_task_new (NULL, NULL, callback, user_data);
+  g_task_set_source_tag (data->task, goa_provider_get_all);
   g_queue_init (&data->ret);
 
   /* Load the normal providers. */
@@ -1249,8 +1186,8 @@ goa_provider_get_all (GAsyncReadyCallback callback,
     {
       GIOExtension *extension = l->data;
       /* The extensions are loaded in the reverse order we used in
-       * goa_provider_ensure_builtins_loaded, so we need to push
-       * extension if front of the already loaded ones. */
+       * ensure_builtins_loaded, so we need to push extension if front of
+       * the already loaded ones. */
       g_queue_push_head (&data->ret, g_object_new (g_io_extension_get_type (extension), NULL));
     }
 
@@ -1287,86 +1224,32 @@ goa_provider_get_all_finish (GList        **out_providers,
                              GAsyncResult  *result,
                              GError       **error)
 {
-  GSimpleAsyncResult *simple = (GSimpleAsyncResult *) result;
+  GTask *task = G_TASK (result);
   GList *providers;
+  gboolean had_error;
 
-  g_return_val_if_fail (g_simple_async_result_is_valid (result, NULL,
-        goa_provider_get_all), FALSE);
+  g_return_val_if_fail (g_task_is_valid (result, NULL), FALSE);
+  g_return_val_if_fail (g_task_get_source_tag (task) == goa_provider_get_all, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  if (g_simple_async_result_propagate_error (simple, error))
+  /* Workaround for bgo#764163 */
+  had_error = g_task_had_error (task);
+  providers = g_task_propagate_pointer (task, error);
+  if (had_error)
     return FALSE;
 
   if (out_providers != NULL)
     {
-      providers = g_simple_async_result_get_op_res_gpointer (simple);
-      *out_providers = g_list_copy_deep (providers, (GCopyFunc) g_object_ref, NULL);
+      *out_providers = providers;
+      providers = NULL;
     }
 
+  g_list_free_full (providers, g_object_unref);
   return TRUE;
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-void
-goa_provider_remove_account (GoaProvider          *self,
-                             GoaObject            *object,
-                             GCancellable         *cancellable,
-                             GAsyncReadyCallback   callback,
-                             gpointer              user_data)
-{
-  g_return_if_fail (GOA_IS_PROVIDER (self));
-  g_return_if_fail (GOA_IS_OBJECT (object));
-  g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
-
-  GOA_PROVIDER_GET_CLASS (self)->remove_account (self, object, cancellable, callback, user_data);
-}
-
-gboolean
-goa_provider_remove_account_finish (GoaProvider   *self,
-                                    GAsyncResult  *res,
-                                    GError       **error)
-{
-  g_return_val_if_fail (GOA_IS_PROVIDER (self), FALSE);
-  g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-  return GOA_PROVIDER_GET_CLASS (self)->remove_account_finish (self, res, error);
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
-static void
-goa_provider_remove_account_real (GoaProvider          *self,
-                                  GoaObject            *object,
-                                  GCancellable         *cancellable,
-                                  GAsyncReadyCallback   callback,
-                                  gpointer              user_data)
-{
-  GTask *task;
-
-  task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_source_tag (task, goa_provider_remove_account_real);
-  g_task_return_boolean (task, TRUE);
-  g_object_unref (task);
-}
-
-static gboolean
-goa_provider_remove_account_finish_real (GoaProvider   *self,
-                                         GAsyncResult  *res,
-                                         GError       **error)
-{
-  GTask *task;
-
-  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
-  task = G_TASK (res);
-
-  g_warn_if_fail (g_task_get_source_tag (task) == goa_provider_remove_account_real);
-
-  return g_task_propagate_boolean (task, error);
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
 /**
  * goa_provider_set_preseed_data:
  * @self: The #GoaProvider
@@ -1595,7 +1478,7 @@ goa_util_add_account_info (GtkGrid *grid, gint row, GoaObject *object)
   gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
   gtk_label_set_max_width_chars (GTK_LABEL (label), 24);
   gtk_label_set_width_chars (GTK_LABEL (label), 24);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
   gtk_widget_set_margin_bottom (label, 12);
   g_free (markup);
   gtk_grid_attach (grid, label, 1, row, 3, 1);
@@ -1643,8 +1526,8 @@ goa_util_add_row_switch_from_keyfile_with_blurb (GtkGrid      *grid,
       GtkWidget *label;
 
       label = gtk_label_new_with_mnemonic (blurb);
-      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), switch_);
+      gtk_label_set_xalign (GTK_LABEL (label), 0.0);
       gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
     }
 
