@@ -385,8 +385,12 @@ goa_utils_keyfile_remove_key (GoaAccount *account, const gchar *key)
 {
   GError *error;
   GKeyFile *key_file;
+  gchar *contents;
   gchar *group;
   gchar *path;
+  gsize length;
+
+  contents = NULL;
 
   path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
   group = g_strdup_printf ("Account %s", goa_account_get_id (account));
@@ -407,11 +411,11 @@ goa_utils_keyfile_remove_key (GoaAccount *account, const gchar *key)
       goto out;
     }
 
-  if (!g_key_file_remove_key (key_file, group, key, NULL))
-    goto out;
+  g_key_file_remove_key (key_file, group, key, NULL);
+  contents = g_key_file_to_data (key_file, &length, NULL);
 
   error = NULL;
-  if (!g_key_file_save_to_file (key_file, path, &error))
+  if (!g_file_set_contents (path, contents, length, &error))
     {
       g_prefix_error (&error, "Error writing key-value-file %s: ", path);
       g_warning ("%s (%s, %d)", error->message, g_quark_to_string (error->domain), error->code);
@@ -420,6 +424,7 @@ goa_utils_keyfile_remove_key (GoaAccount *account, const gchar *key)
     }
 
  out:
+  g_free (contents);
   g_key_file_free (key_file);
   g_free (group);
   g_free (path);
@@ -430,10 +435,12 @@ goa_utils_keyfile_set_boolean (GoaAccount *account, const gchar *key, gboolean v
 {
   GError *error;
   GKeyFile *key_file;
-  gboolean needs_update = FALSE;
-  gboolean old_value;
+  gchar *contents;
   gchar *group;
   gchar *path;
+  gsize length;
+
+  contents = NULL;
 
   path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
   group = g_strdup_printf ("Account %s", goa_account_get_id (account));
@@ -454,31 +461,11 @@ goa_utils_keyfile_set_boolean (GoaAccount *account, const gchar *key, gboolean v
       goto out;
     }
 
-  error = NULL;
-  old_value = g_key_file_get_boolean (key_file, group, key, &error);
-  if (error != NULL)
-    {
-      g_warning ("Error reading key %s from keyfile %s: %s (%s, %d)",
-                 key,
-                 path,
-                 error->message,
-                 g_quark_to_string (error->domain),
-                 error->code);
-      needs_update = TRUE;
-      g_error_free (error);
-    }
-  else if (old_value != value)
-    {
-      needs_update = TRUE;
-    }
-
-  if (!needs_update)
-    goto out;
-
   g_key_file_set_boolean (key_file, group, key, value);
+  contents = g_key_file_to_data (key_file, &length, NULL);
 
   error = NULL;
-  if (!g_key_file_save_to_file (key_file, path, &error))
+  if (!g_file_set_contents (path, contents, length, &error))
     {
       g_prefix_error (&error, "Error writing key-value-file %s: ", path);
       g_warning ("%s (%s, %d)", error->message, g_quark_to_string (error->domain), error->code);
@@ -487,6 +474,7 @@ goa_utils_keyfile_set_boolean (GoaAccount *account, const gchar *key, gboolean v
     }
 
  out:
+  g_free (contents);
   g_key_file_free (key_file);
   g_free (group);
   g_free (path);
@@ -497,10 +485,12 @@ goa_utils_keyfile_set_string (GoaAccount *account, const gchar *key, const gchar
 {
   GError *error;
   GKeyFile *key_file;
-  gboolean needs_update = FALSE;
+  gchar *contents;
   gchar *group;
-  gchar *old_value = NULL;
   gchar *path;
+  gsize length;
+
+  contents = NULL;
 
   path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
   group = g_strdup_printf ("Account %s", goa_account_get_id (account));
@@ -521,31 +511,11 @@ goa_utils_keyfile_set_string (GoaAccount *account, const gchar *key, const gchar
       goto out;
     }
 
-  error = NULL;
-  old_value = g_key_file_get_string (key_file, group, key, &error);
-  if (error != NULL)
-    {
-      g_warning ("Error reading key %s from keyfile %s: %s (%s, %d)",
-                 key,
-                 path,
-                 error->message,
-                 g_quark_to_string (error->domain),
-                 error->code);
-      needs_update = TRUE;
-      g_error_free (error);
-    }
-  else if (g_strcmp0 (old_value, value) != 0)
-    {
-      needs_update = TRUE;
-    }
-
-  if (!needs_update)
-    goto out;
-
   g_key_file_set_string (key_file, group, key, value);
+  contents = g_key_file_to_data (key_file, &length, NULL);
 
   error = NULL;
-  if (!g_key_file_save_to_file (key_file, path, &error))
+  if (!g_file_set_contents (path, contents, length, &error))
     {
       g_prefix_error (&error, "Error writing key-value-file %s: ", path);
       g_warning ("%s (%s, %d)", error->message, g_quark_to_string (error->domain), error->code);
@@ -554,9 +524,9 @@ goa_utils_keyfile_set_string (GoaAccount *account, const gchar *key, const gchar
     }
 
  out:
+  g_free (contents);
   g_key_file_free (key_file);
   g_free (group);
-  g_free (old_value);
   g_free (path);
 }
 
@@ -676,7 +646,7 @@ goa_utils_get_credentials (GoaProvider    *provider,
   GVariant *credentials = NULL;
   GoaAccount *account = NULL;
   gboolean ret = FALSE;
-  gchar *username = NULL;
+  const gchar *username;
   gchar *password = NULL;
 
   credentials = goa_utils_lookup_credentials_sync (provider,
@@ -687,7 +657,7 @@ goa_utils_get_credentials (GoaProvider    *provider,
     goto out;
 
   account = goa_object_get_account (object);
-  username = goa_account_dup_identity (account);
+  username = goa_account_get_identity (account);
 
   if (!g_variant_lookup (credentials, id, "s", &password))
     {
@@ -698,10 +668,7 @@ goa_utils_get_credentials (GoaProvider    *provider,
     }
 
   if (out_username)
-    {
-      *out_username = username;
-      username = NULL;
-    }
+    *out_username = g_strdup (username);
 
   if (out_password)
     {
@@ -714,7 +681,6 @@ goa_utils_get_credentials (GoaProvider    *provider,
 out:
   g_clear_object (&account);
   g_clear_pointer (&credentials, (GDestroyNotify) g_variant_unref);
-  g_free (username);
   g_free (password);
   return ret;
 }
