@@ -19,7 +19,7 @@
 #include "config.h"
 #include "goaidentityservice.h"
 
-#include <glib/gi18n-lib.h>
+#include <glib/gi18n.h>
 #include <gmodule.h>
 
 #include <glib.h>
@@ -48,6 +48,7 @@ struct _GoaIdentityServicePrivate
   GHashTable               *pending_temporary_account_results;
 
   GoaClient                *client;
+  GoaManager               *accounts_manager;
 };
 
 static void identity_service_manager_interface_init (GoaIdentityServiceManagerIface *interface);
@@ -303,8 +304,6 @@ on_sign_in_handled (GoaIdentityService    *self,
                                                      invocation,
                                                      object_path);
     }
-
-  g_object_unref (invocation);
 }
 
 static void
@@ -390,7 +389,7 @@ goa_identity_service_handle_sign_in (GoaIdentityServiceManager *manager,
                                                 (GAsyncReadyCallback)
                                                 on_sign_in_handled,
                                                 g_object_ref (invocation),
-                                                NULL);
+                                                g_strdup (identifier));
   cancellable = g_cancellable_new ();
   g_object_set_data (G_OBJECT (operation_result),
                      "cancellable",
@@ -432,8 +431,6 @@ on_sign_out_handled (GoaIdentityService    *self,
   else
     goa_identity_service_manager_complete_sign_out (GOA_IDENTITY_SERVICE_MANAGER (self),
                                                     invocation);
-
-  g_object_unref (invocation);
 }
 
 static void
@@ -445,7 +442,7 @@ on_identity_signed_out (GoaIdentityManager *manager,
   GError             *error;
   GoaIdentity        *identity;
   const char         *identifier;
-  GoaObject          *object = NULL;
+  GoaObject          *object;
 
   error = NULL;
   goa_identity_manager_sign_identity_out_finish (manager, result, &error);
@@ -467,8 +464,6 @@ on_identity_signed_out (GoaIdentityManager *manager,
     ensure_account_credentials (self, object);
 
   g_simple_async_result_complete_in_idle (operation_result);
-
-  g_clear_object (&object);
   g_object_unref (operation_result);
 }
 
@@ -478,7 +473,7 @@ on_got_identity_for_sign_out (GoaIdentityManager *manager,
                               GSimpleAsyncResult *operation_result)
 {
   GError *error;
-  GoaIdentity *identity = NULL;
+  GoaIdentity *identity;
 
   error = NULL;
   identity = goa_identity_manager_get_identity_finish (manager, result, &error);
@@ -487,9 +482,7 @@ on_got_identity_for_sign_out (GoaIdentityManager *manager,
     {
       g_debug ("GoaIdentityService: Identity could not be signed out: %s",
                error->message);
-      g_simple_async_result_take_error (operation_result, error);
-      g_simple_async_result_complete_in_idle (operation_result);
-      goto out;
+      return;
     }
 
   g_object_set_data_full (G_OBJECT (operation_result),
@@ -503,11 +496,7 @@ on_got_identity_for_sign_out (GoaIdentityManager *manager,
                                           NULL,
                                           (GAsyncReadyCallback)
                                           on_identity_signed_out,
-                                          g_object_ref (operation_result));
-
- out:
-  g_clear_object (&identity);
-  g_object_unref (operation_result);
+                                          operation_result);
 }
 
 static gboolean
@@ -556,8 +545,6 @@ on_secret_keys_exchanged (GoaIdentityService *self,
                                                                   invocation,
                                                                   output_key);
     }
-
-  g_object_unref (invocation);
 }
 
 static void
@@ -586,6 +573,7 @@ on_caller_watched (GDBusConnection    *connection,
                                        GCR_ERROR_UNRECOGNIZED,
                                        _("Initial secret key is invalid"));
       g_simple_async_result_complete_in_idle (operation_result);
+      g_object_unref (operation_result);
       return;
     }
 
@@ -666,7 +654,6 @@ goa_identity_service_handle_exchange_secret_keys (GoaIdentityServiceManager *man
                        g_strdup (sender),
                        GUINT_TO_POINTER (watch_id));
 
-  g_object_unref (operation_result);
   return TRUE;
 }
 
@@ -760,7 +747,7 @@ on_identity_needs_renewal (GoaIdentityManager *identity_manager,
                            GoaIdentityService *self)
 {
   const char *principal;
-  GoaObject  *object = NULL;
+  GoaObject  *object;
 
   principal = goa_identity_get_identifier (identity);
 
@@ -769,7 +756,8 @@ on_identity_needs_renewal (GoaIdentityManager *identity_manager,
   if (object != NULL && should_ignore_object (self, object))
     {
       g_debug ("GoaIdentityService: ignoring identity %s that says it needs renewal", principal);
-      goto out;
+
+      return;
     }
 
   g_debug ("GoaIdentityService: identity %s needs renewal", principal);
@@ -781,9 +769,6 @@ on_identity_needs_renewal (GoaIdentityManager *identity_manager,
                                        (GAsyncReadyCallback)
                                        on_identity_renewed,
                                        self);
-
- out:
-  g_clear_object (&object);
 }
 
 static void
@@ -907,7 +892,6 @@ add_temporary_account (GoaIdentityService *self,
   GSimpleAsyncResult *operation_result;
   GVariantBuilder     credentials;
   GVariantBuilder     details;
-  GoaManager         *manager;
   GoaObject *object;
 
   principal = goa_identity_get_identifier (identity);
@@ -950,8 +934,7 @@ add_temporary_account (GoaIdentityService *self,
                        g_strdup (principal),
                        g_object_ref (operation_result));
 
-  manager = goa_client_get_manager (self->priv->client);
-  goa_manager_call_add_account (manager,
+  goa_manager_call_add_account (self->priv->accounts_manager,
                                 "kerberos",
                                 principal,
                                 principal,
@@ -981,8 +964,6 @@ on_identity_added (GoaIdentityManager *identity_manager,
 
   if (object == NULL)
     add_temporary_account (self, identity);
-
-  g_clear_object (&object);
 }
 
 static void
@@ -1000,7 +981,6 @@ on_identity_removed (GoaIdentityManager *identity_manager,
     ensure_account_credentials (self, object);
 
   unexport_identity (self, identity);
-  g_clear_object (&object);
 }
 
 static void
@@ -1018,8 +998,6 @@ on_identity_refreshed (GoaIdentityManager *identity_manager,
     add_temporary_account (self, identity);
   else
     ensure_account_credentials (self, object);
-
-  g_clear_object (&object);
 }
 
 typedef struct
@@ -1360,7 +1338,6 @@ on_identity_expiring (GoaIdentityManager *identity_manager,
     return;
 
   ensure_account_credentials (self, object);
-  g_clear_object (&object);
 }
 
 static void
@@ -1381,7 +1358,6 @@ on_identity_expired (GoaIdentityManager *identity_manager,
     return;
 
   ensure_account_credentials (self, object);
-  g_clear_object (&object);
 }
 
 static void
@@ -1554,6 +1530,44 @@ on_account_interface_removed (GDBusObjectManager *manager,
                                      result);
 }
 
+static void
+on_account_removed (GoaClient          *client,
+                    GoaObject          *object,
+                    GoaIdentityService *self)
+{
+  GSimpleAsyncResult *result;
+  GoaAccount         *account;
+  const char         *provider_type;
+  const char         *account_identity;
+
+  account = goa_object_peek_account (object);
+
+  if (account == NULL)
+    return;
+
+  provider_type = goa_account_get_provider_type (account);
+
+  if (g_strcmp0 (provider_type, "kerberos") != 0)
+    return;
+
+  account_identity = goa_account_get_identity (account);
+
+  g_debug ("Kerberos account %s removed and should now be signed out", account_identity);
+
+  result = g_simple_async_result_new (G_OBJECT (self),
+                                      (GAsyncReadyCallback)
+                                      on_sign_out_for_account_change_done,
+                                      NULL,
+                                      on_account_removed);
+
+  goa_identity_manager_get_identity (self->priv->identity_manager,
+                                     account_identity,
+                                     NULL,
+                                     (GAsyncReadyCallback)
+                                     on_got_identity_for_sign_out,
+                                     result);
+}
+
 static void
 on_identities_listed (GoaIdentityManager *manager,
                       GAsyncResult       *result,
@@ -1587,6 +1601,11 @@ on_identities_listed (GoaIdentityManager *manager,
                     G_CALLBACK (on_identity_expired),
                     self);
 
+  g_signal_connect (G_OBJECT (self->priv->client),
+                    "account-removed",
+                    G_CALLBACK (on_account_removed),
+                    self);
+
   identities = goa_identity_manager_list_identities_finish (manager, result, &error);
 
   if (identities == NULL)
@@ -1668,8 +1687,6 @@ ensure_credentials_for_accounts (GoaIdentityService *self)
 
       ensure_account_credentials (self, object);
     }
-
-  g_list_free_full (accounts, g_object_unref);
 }
 
 static void
@@ -1689,6 +1706,8 @@ on_got_client (GoaClient          *client,
       goto out;
     }
 
+  self->priv->accounts_manager = goa_client_get_manager (client);
+
   self->priv->identity_manager = goa_kerberos_identity_manager_new (NULL, &error);
 
   if (self->priv->identity_manager == NULL)
